/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: PortViewer.java,v 1.8 2007/05/21 17:20:11 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import xsul5.XmlConstants;
import edu.indiana.extreme.xbaya.graph.DataPort;
import edu.indiana.extreme.xbaya.graph.Port;
import edu.indiana.extreme.xbaya.graph.gui.GraphCanvas;
import edu.indiana.extreme.xbaya.graph.gui.GraphCanvasEvent;
import edu.indiana.extreme.xbaya.graph.gui.GraphCanvasListener;
import edu.indiana.extreme.xbaya.graph.gui.GraphCanvasEvent.GraphCanvasEventType;
import edu.indiana.extreme.xbaya.util.SwingUtil;

/**
 * @author Satoshi Shirasuna
 */
public class PortViewer implements GraphCanvasListener {

    /**
     * The title.
     */
    public static final String TITLE = "Parameters";

    private JPanel panel;

    private JEditorPane outputEditor;

    private JEditorPane inputEditor;

    /**
     * Creates a PortInformation
     */
    public PortViewer() {
        this.panel = new JPanel();
        this.panel.setMinimumSize(SwingUtil.MINIMUM_SIZE);
        this.panel.setPreferredSize(new Dimension(0, 150));

        JPanel inBox = new JPanel(new BorderLayout());
        inBox
                .setBorder(new TitledBorder(new EtchedBorder(),
                        "Input Parameter"));
        this.inputEditor = createEditorPane();
        JScrollPane inputScrollPane = new JScrollPane(this.inputEditor);
        inputScrollPane.setMinimumSize(SwingUtil.MINIMUM_SIZE);
        inBox.add(inputScrollPane, BorderLayout.CENTER);
        inBox.setMinimumSize(SwingUtil.MINIMUM_SIZE);

        JPanel outBox = new JPanel(new BorderLayout());
        outBox.setBorder(new TitledBorder(new EtchedBorder(),
                "Output Parameter"));
        this.outputEditor = createEditorPane();
        JScrollPane outScrollPane = new JScrollPane(this.outputEditor);
        outScrollPane.setMinimumSize(SwingUtil.MINIMUM_SIZE);
        outBox.add(outScrollPane, BorderLayout.CENTER);
        outBox.setMinimumSize(SwingUtil.MINIMUM_SIZE);

        this.panel.setLayout(new GridLayout(1, 2));
        this.panel.add(outBox);
        this.panel.add(inBox);
    }

    /**
     * @return The panel
     */
    public JComponent getSwingComponent() {
        return this.panel;
    }

    /**
     * @param port
     */
    public void setOutputPort(Port port) {
        showPortInfo(this.outputEditor, port);
    }

    /**
     * @param port
     */
    public void setInputPort(Port port) {
        showPortInfo(this.inputEditor, port);
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.gui.GraphCanvasListener#graphCanvasChanged(edu.indiana.extreme.xbaya.graph.gui.GraphCanvasEvent)
     */
    public void graphCanvasChanged(GraphCanvasEvent event) {
        GraphCanvasEventType type = event.getType();
        GraphCanvas graphCanvas = event.getGraphCanvas();
        switch (type) {
        case INPUT_PORT_SELECTED:
            Port inputPort = graphCanvas.getSelectedInputPort();
            setInputPort(inputPort);
            break;
        case OUTPUT_PORT_SELECTED:
            Port outputPort = graphCanvas.getSelectedOutputPort();
            setOutputPort(outputPort);
            break;
        case GRAPH_LOADED:
        case NAME_CHANGED:
        case NODE_SELECTED:
            // do nothing
        }
    }

    /**
     * Shows the information of a selected port on the list specified.
     * 
     * @param editor
     * @param port
     */
    private void showPortInfo(JEditorPane editor, Port port) {
        if (port == null) {
            editor.setText("");
        } else {
            // TODO dispatch to each port?.
            StringBuilder buf = new StringBuilder();
            buf.append("<strong>Component: " + port.getNode().getName()
                    + "</strong><br>");
            buf.append("<strong>Port: " + port.getName() + "<br></strong>");
            if (port instanceof DataPort) {
                buf.append("<strong>Type</strong>: "
                        + ((DataPort) port).getType() + "<br>");
            } else {
                // TODO
            }
            buf.append("<strong>Description</strong>: "
                    + port.getComponentPort().getDescription() + "<br>");
            editor.setText(buf.toString());

            // To prevent from scrolling down to the bottom.
            editor.setCaretPosition(0);
        }
    }

    private JEditorPane createEditorPane() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setMinimumSize(SwingUtil.MINIMUM_SIZE);
        editorPane.setEditable(false);
        editorPane.setBackground(Color.WHITE);
        editorPane.setContentType(XmlConstants.CONTENT_TYPE_HTML);
        return editorPane;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
